@@ -2,7 +2,7 @@
 # ABSTRACT: Find runs and tests according to user specifications.
 
 package TestRail::Utils::Find;
-$TestRail::Utils::Find::VERSION = '0.039';
+$TestRail::Utils::Find::VERSION = '0.040';
 use strict;
 use warnings;
 
@@ -165,7 +165,10 @@ sub findTests {
         }
         foreach my $case (@cases) {
             foreach my $path (@realtests) {
-                next unless $case->{'title'} eq basename($path);
+
+                #Filter obviously bogus stuff first to not incur basename() cost except for when we're right, or have a name that contains this name
+                next unless index( $path, $case->{'title'} ) > 0;
+                next unless basename($path) eq $case->{title};
                 $case->{'path'} = $path;
                 push( @tmpArr, $case );
                 last;
@@ -286,6 +289,10 @@ sub getResults {
           if $opts->{projects}
           && !( grep { $_ eq $project->{'name'} } @{ $opts->{'projects'} } );
         my $runs = $tr->getRuns( $project->{'id'} );
+
+        #XXX No runs, or temporary error to ignore
+        next unless ref($runs) eq 'ARRAY';
+
         push( @seenRunIds, map { $_->{id} } @$runs );
 
         #Translate plan names to ids
@@ -345,9 +352,16 @@ sub getResults {
             @results,
             mce_loop {
                 my $runz = $_;
-                my $res  = {};
+
+                #XXX it appears as though some versions of MCE do not have uniform passing convention
+                $runz = [$runz] if ref($runz) ne 'ARRAY';
+
+                my $res = {};
                 foreach my $run (@$runz) {
 
+                    #XXX super bad bug in some versions of MCE, apparently causes data loss, or is duping jobs with incomplete info!
+                    next if !$run->{id};
+
                     #Translate config ids to names, also remove any gone configs
                     my @run_configs =
                       grep { defined $_ }
@@ -356,7 +370,6 @@ sub getResults {
                       if scalar( @{ $opts->{runs} } )
                       && !( grep { $_ eq $run->{'name'} }
                         @{ $opts->{'runs'} } );
-
                     if ( $opts->{fast} ) {
                         my @csz = @cases;
                         @csz = grep { ref($_) eq 'HASH' } map {
@@ -405,6 +418,7 @@ sub getResults {
             }
             @$runs
         );
+
     }
 
     foreach my $result (@results) {
@@ -466,7 +480,7 @@ TestRail::Utils::Find - Find runs and tests according to user specifications.
 
 =head1 VERSION
 
-version 0.039
+version 0.040
 
 =head1 DESCRIPTION
 
